import re
import json
import os
import pytest
import requests


def ignore_alert(alert):
    """
    Returns true if an alert should be ignored. This is when the "severity"
    equals "none", or in some application specific cases.
    """

    # Ignore watchdog alerts
    if alert["labels"]["severity"] == "none":
        return True

    # Ignore `KubeAPILatencyHigh` fom high load during installation
    # phase
    if re.search(r"(KubeAPIDown|KubeletDown)", alert["labels"]["alertname"]):
        return True

    return False


@pytest.mark.prometheus
def test_prometheus_alerts(host):
    """Test prometheus for firing alerts."""

    def summarize_alerts(alerts):
        """Print a alert summary."""
        print('Total alerts: %s' % len(alerts))
        print(json.dumps(alerts, indent=2))

    print("Starting prometheus test...")

    domain = os.environ.get("FQDN")
    if domain:
        print(f"Using domain {domain} from FQDN environment variable.")
    else:
        ansible_vars = host.ansible.get_variables()
        domain = ansible_vars["domain"]
        print(f"Using domain {domain} from ansible inventory.")
    url = f'https://prometheus.{domain}/api/v1/alerts'
    print(f'Querying {url}')

    basic_auth_pw = os.environ.get("BASIC_AUTH_PW")
    assert basic_auth_pw, 'Please export BASIC_AUTH_PW env var!'

    resp = requests.get(url, auth=('admin', basic_auth_pw))
    assert resp.status_code == 200
    alert_json = json.loads(resp.content)
    status = alert_json["status"]
    alerts = alert_json["data"]["alerts"]

    real_alerts = []
    ignored_alerts = []

    for alert in alerts:
        # Filter out the ever firing "Dead mans switch" test alert
        if ignore_alert(alert):
            ignored_alerts.append(alert)
        else:
            real_alerts.append(alert)

    print('\n\n\n========= Ignored ==========')
    summarize_alerts(ignored_alerts)

    print('\n\n\n========= Firing ==========')
    summarize_alerts(real_alerts)

    count = len(real_alerts)

    assert status == "success", "Failure queriying the prometheus api at" + url
    assert count == 0, "Firing alerts: {0}".format(count)


if __name__ == "__main__":
    test_prometheus_alerts('')
